@@ -37,7 +37,7 @@ _COLOR_MAP = {
 }
 
 
-def _dump_tree(obj: docspec._Base, depth: int = 0):
+def _dump_tree(obj: docspec.ApiObject, depth: int = 0):
   color = _COLOR_MAP.get(type(obj))
   type_name = colored(type(obj).__name__.lower(), color)
   print('| ' * depth + type_name, obj.name)
@@ -49,7 +49,7 @@ def main():
   parser = argparse.ArgumentParser()
   parser.add_argument('file', nargs='?')
   parser.add_argument('-t', '--tty', action='store_true', help='Read from stdin even if it is a TTY.')
-  parser.add_argument('--multiple', action='store_true', help='Load a module per line from the input.')
+  parser.add_argument('-m', '--multiple', action='store_true', help='Load a module per line from the input.')
   parser.add_argument('--dump-tree', action='store_true', help='Dump a simplified tree representation of the parsed module(s) to stdout. Supports colored output if the "termcolor" module is installed.')
   args = parser.parse_args()
 
